@@ -2,20 +2,35 @@ source("data.R")
 
 #### aggregate method ####
 expect_identical(
-  DTSg$new(DT1)$aggregate(byYmdH__, mean)$values(),
+  DTSg$new(DT1)$aggregate(byYmdH__, mean)$values(TRUE),
   data.table(
-    date = seq(
+    .dateTime = seq(
       as.POSIXct("2000-10-29 01:00:00", tz = "Europe/Vienna"),
       as.POSIXct("2000-10-29 03:30:00", tz = "Europe/Vienna"),
       "1 hour"
     ),
     col1 = c( 2, 6, 10, 14),
     col2 = c(NA, 6, 10, 14),
-    key = "date"
+    key = ".dateTime"
   ),
   info = "values are aggregated correctly (multiple columns and single function)"
 )
 
+expect_identical(
+  DTSg$new(DT1)$aggregate(byYmdH__, "mean")$values(TRUE),
+  data.table(
+    .dateTime = seq(
+      as.POSIXct("2000-10-29 01:00:00", tz = "Europe/Vienna"),
+      as.POSIXct("2000-10-29 03:30:00", tz = "Europe/Vienna"),
+      "1 hour"
+    ),
+    col1 = c( 2, 6, 10, 14),
+    col2 = c(NA, 6, 10, 14),
+    key = ".dateTime"
+  ),
+  info = "values are aggregated correctly (multiple columns and single character function)"
+)
+
 expect_identical(
   DTSg$new(DT1)$aggregate(byYmdH__, mean, cols = "col2", n = TRUE)$values(TRUE)[[".n"]],
   c(1L, 2L, 2L, 2L),
@@ -23,9 +38,15 @@ expect_identical(
 )
 
 expect_identical(
-  DTSg$new(DT1)$aggregate(byYmdH__, list(mean = mean, sum = sum), n = TRUE)$values(),
+  DTSg$new(DT1)$aggregate(byYmdH__, "mean", cols = "col2", n = TRUE)$values(TRUE)[[".n"]],
+  c(1L, 2L, 2L, 2L),
+  info = ".n is correct (single column and character function)"
+)
+
+expect_identical(
+  DTSg$new(DT1)$aggregate(byYmdH__, list(mean = mean, sum = sum), n = TRUE)$values(TRUE),
   data.table(
-    date = seq(
+    .dateTime = seq(
       as.POSIXct("2000-10-29 01:00:00", tz = "Europe/Vienna"),
       as.POSIXct("2000-10-29 03:30:00", tz = "Europe/Vienna"),
       "1 hour"
@@ -35,11 +56,29 @@ expect_identical(
     col2.mean = c(NA, 6 , 10, 14),
     col2.sum  = c(NA, 12, 20, 28),
     .n = rep(2L, 4),
-    key = "date"
+    key = ".dateTime"
   ),
   info = "values are aggregated correctly and .n is correct (multiple columns and functions)"
 )
 
+expect_identical(
+  DTSg$new(DT1)$aggregate(byYmdH__, c(mean = "mean", sum = "sum"), n = TRUE)$values(TRUE),
+  data.table(
+    .dateTime = seq(
+      as.POSIXct("2000-10-29 01:00:00", tz = "Europe/Vienna"),
+      as.POSIXct("2000-10-29 03:30:00", tz = "Europe/Vienna"),
+      "1 hour"
+    ),
+    col1.mean = c( 2, 6 , 10, 14),
+    col1.sum  = c( 4, 12, 20, 28),
+    col2.mean = c(NA, 6 , 10, 14),
+    col2.sum  = c(NA, 12, 20, 28),
+    .n = rep(2L, 4),
+    key = ".dateTime"
+  ),
+  info = "values are aggregated correctly and .n is correct (multiple columns and character functions)"
+)
+
 expect_identical(
   DTSg$new(DT1)$aggregate(byYmdH__, mean, na.rm = TRUE)$values(TRUE)[["col2"]],
   c(1, 6, 10, 14),
@@ -47,11 +86,31 @@ expect_identical(
 )
 
 expect_identical(
-  DTSg$new(DT1)$aggregate(byYmdH__, list(mean = mean, sum = sum), na.rm = TRUE)$values(TRUE)[["col2.sum"]],
+  DTSg$new(DT1)$aggregate(byYmdH__, "mean", na.rm = TRUE)$values(TRUE)[["col2"]],
+  c(1, 6, 10, 14),
+  info = '"..." passes on arguments correctly (single character function)'
+)
+
+expect_identical(
+  DTSg$new(DT1)$aggregate(
+    byYmdH__,
+    list(mean = mean, sum = sum),
+    na.rm = TRUE
+  )$values(TRUE)[["col2.sum"]],
   c(1, 12, 20, 28),
   info = '"..." passes on arguments correctly (multiple functions)'
 )
 
+expect_identical(
+  DTSg$new(DT1)$aggregate(
+    byYmdH__,
+    c(mean = "mean", sum = "sum"),
+    na.rm = TRUE
+  )$values(TRUE)[["col2.sum"]],
+  c(1, 12, 20, 28),
+  info = '"..." passes on arguments correctly (multiple character functions)'
+)
+
 expect_true(
   DTSg$new(DT1)$aggregate(byYmdH__, mean)$aggregated,
   info = '"aggregated" field is set correctly'
@@ -59,9 +118,13 @@ expect_true(
 
 #### alter method ####
 expect_identical(
-  DTSg$new(DT1[-3L, ])$alter("2000-10-29 01:00:00", "2000-10-29 03:00:00", "1 hour")$values(),
+  DTSg$new(DT1[-3L, ])$alter(
+    "2000-10-29 01:00:00",
+    "2000-10-29 03:00:00",
+    "1 hour"
+  )$values(TRUE),
   data.table(
-    date = seq(
+    .dateTime = seq(
       as.POSIXct("2000-10-29 01:00:00", tz = "Europe/Vienna"),
       as.POSIXct("2000-10-29 03:00:00", tz = "Europe/Vienna"),
       "1 hour"
@@ -69,13 +132,29 @@ expect_identical(
     col1 = c(  1, NA,   9,  13),
     col2 = c(  1, NA,   9,  13),
     col3 = c("A", NA, "E", "G"),
-    key = "date"
+    key = ".dateTime"
   ),
-  info = "values are altered correctly (explicit missing values)"
+  info = "values are altered correctly (explicitly missing values)"
 )
 
 expect_identical(
-  DTSg$new(DT1[-3L, ])$alter("2000-01-31", "2000-07-01", "1 month")$values()[["date"]],
+  DTSg$new(DT2[, .(date, col1)])$alter(na.status = "implicit")$values(),
+  setkey(DT2[3L, .(date, col1)], "date"),
+  info = "values are altered correctly (single column and implicitly missing values)"
+)
+
+expect_identical(
+  DTSg$new(DT2[, -4L])$alter(na.status = "implicit")$values(),
+  setkey(DT2[3L, -4L], "date"),
+  info = "values are altered correctly (multiple columns and implicitly missing values)"
+)
+
+expect_identical(
+  DTSg$new(DT1[-3L, ])$alter(
+    "2000-01-31",
+    "2000-07-01",
+    "1 month"
+  )$values(TRUE)[[".dateTime"]],
   as.POSIXct(
     c("2000-01-31", "2000-02-29", "2000-03-31", "2000-04-30", "2000-05-31", "2000-06-30"),
     tz = "Europe/Vienna"
@@ -90,19 +169,7 @@ expect_error(
 
 expect_warning(
   DTSg$new(DT3),
-  info = "explicit missing values and unrecognised periodicity returns warning"
-)
-
-expect_identical(
-  DTSg$new(DT2[, .(date, col1)])$alter(na.status = "implicit")$values(),
-  setkey(DT2[3L, .(date, col1)], "date"),
-  info = "values are altered correctly (single column and implicit missing values)"
-)
-
-expect_identical(
-  DTSg$new(DT2[, -4L, with = FALSE])$alter(na.status = "implicit")$values(),
-  setkey(DT2[3L, -4L, with = FALSE], "date"),
-  info = "values are altered correctly (multiple columns and implicit missing values)"
+  info = "explicitly missing values and unrecognised periodicity returns warning"
 )
 
 expect_error(
@@ -111,14 +178,14 @@ expect_error(
 )
 
 expect_identical(
-  DTSg$new(DT1, na.status = "undecided")$na.status,
-  "undecided",
+  DTSg$new(DT1)$na.status,
+  "explicit",
   info = '"na.status" field is set correctly'
 )
 
 expect_identical(
   {
-    TS <- DTSg$new(DT1, na.status = "explicit")
+    TS <- DTSg$new(DT1, na.status = "undecided")
     TS$na.status <- "implicit"
     TS$na.status
   },
@@ -145,6 +212,8 @@ expect_identical(
     TS$colapply(function(x, ...) {as.factor(x)})
     TS$merge(DT2)
     TS$rollapply(weighted.mean, na.rm = TRUE)
+    TS$setCols(values = 1)
+    TS$subset(1L)
     TS
   },
   TS,
@@ -212,7 +281,7 @@ expect_identical(
 expect_identical(
   DTSg$new(DT1)$cols("character"),
   "col3",
-  info = "character column name is returned"
+  info = "character column names are returned"
 )
 
 expect_identical(
@@ -230,7 +299,7 @@ expect_identical(
 expect_identical(
   DTSg$new(DT1)$cols(pattern = "^c.l1$"),
   "col1",
-  info = "column name matching pattern is returned"
+  info = "column names matching pattern are returned"
 )
 
 expect_identical(
@@ -254,13 +323,27 @@ expect_error(
 expect_identical(
   DTSg$new(DT1)$getCol("col2"),
   DT1[["col2"]],
-  info = "column is returned"
+  info = "column is returned correctly (getCol)"
+)
+
+expect_identical(
+  {
+    TS <- DTSg$new(DT1)
+    TS["col2"]
+  },
+  DT1[["col2"]],
+  info = "column is returned correctly ([)"
 )
 
 #### initialize method ####
+expect_error(
+  DTSg$new(data.table(date = numeric(0L), col1 = integer(0L))),
+  info = "data.table without a single row returns error"
+)
+
 expect_error(
   DTSg$new(DT1[, .(date)]),
-  info = "data.table with a single column only returns error"
+  info = "data.table with a single column returns error"
 )
 
 expect_true(
@@ -311,11 +394,6 @@ expect_true(
 )
 
 #### merge method ####
-expect_error(
-  DTSg$new(DT1[, .(date, col1)])$merge(DT2, by = "date"),
-  info = "use of arguments not allowed returns error"
-)
-
 expect_identical(
   DTSg$new(DT1[, .(date, col1)])$merge(DT2)$values(),
   merge(DT1[, .(date, col1)], DT2, by = "date"),
@@ -323,9 +401,20 @@ expect_identical(
 )
 
 expect_identical(
-  DTSg$new(DT1[, .(date, col1)])$merge(DT2, all = TRUE)$values(),
-  merge(DT1[, .(date, col1)], DT2, by = "date", all = TRUE),
-  info = '"..." passes on arguments correctly'
+  DTSg$new(DT1[1:4, .(date, col1)])$merge(
+    DTSg$new(DT1[c(5L, 7L, 8L), .(date, col1)], na.status = "implicit"),
+    all = TRUE
+  )$values(),
+  setkey(rbind(
+    DT1[1:4, .(date, col1.x = col1, col1.y = NA                  )],
+    DT1[5:8, .(date, col1.x = NA  , col1.y = col1[c(1, NA, 3, 4)])]
+  ), "date"),
+  info = '"..." passes on arguments correctly and missing values are made explicit'
+)
+
+expect_error(
+  DTSg$new(DT1[, .(date, col1)])$merge(DT2, by = "date"),
+  info = "use of arguments not allowed returns error"
 )
 
 #### nas method ####
@@ -382,10 +471,7 @@ expect_warning(
 )
 
 expect_error(
-  DTSg$new(data.table(
-    date = .POSIXct(NA),
-    col1 = pi
-  )),
+  DTSg$new(data.table(date = .POSIXct(NA_real_), col1 = pi)),
   info = "data.table with a single row and missing timestamp returns error"
 )
 
@@ -470,7 +556,7 @@ expect_identical(
   {
     TS <- DTSg$new(DT1)
     TS$timezone <- "Europe/Kiev"
-    TS$values()[["date"]]
+    TS$values(TRUE)[[".dateTime"]]
   },
   seq(
     as.POSIXct("2000-10-29 02:00:00", tz = "Europe/Kiev"),
@@ -482,37 +568,69 @@ expect_identical(
 
 #### rollapply method ####
 expect_identical(
-  DTSg$new(DT1)$rollapply(function(x, ...) {sum(x)}, before = 2L, after = 1L)$values(TRUE)[["col1"]],
+  DTSg$new(DT1)$rollapply(
+    function(x, ...) {sum(x)},
+    before = 2L,
+    after = 1L,
+    memoryOverCPU = TRUE
+  )$values(TRUE)[["col1"]],
   c(NA, NA, 16, 24, 32, 40, 48, NA),
   info = '"fun" is applied correctly (memoryOverCPU = TRUE)'
 )
 
 expect_identical(
-  DTSg$new(DT1)$rollapply(function(x, ...) {sum(x)}, before = 2L, after = 1L, memoryOverCPU = FALSE)$values(TRUE)[["col1"]],
+  DTSg$new(DT1)$rollapply(
+    function(x, ...) {sum(x)},
+    before = 2L,
+    after = 1L,
+    memoryOverCPU = FALSE
+  )$values(TRUE)[["col1"]],
   c(NA, NA, 16, 24, 32, 40, 48, NA),
   info = '"fun" is applied correctly (memoryOverCPU = FALSE)'
 )
 
 expect_identical(
-  DTSg$new(DT1)$rollapply(mean, na.rm = TRUE, before = 2L, after = 1L)$values(TRUE)[["col1"]],
+  DTSg$new(DT1)$rollapply(
+    mean,
+    na.rm = TRUE,
+    before = 2L,
+    after = 1L,
+    memoryOverCPU = TRUE
+  )$values(TRUE)[["col1"]],
   c(2, 3, 4, 6, 8, 10, 12, 13),
   info = '"..." passes on arguments correctly (memoryOverCPU = TRUE)'
 )
 
 expect_identical(
-  DTSg$new(DT1)$rollapply(mean, na.rm = TRUE, before = 2L, after = 1L, memoryOverCPU = FALSE)$values(TRUE)[["col1"]],
+  DTSg$new(DT1)$rollapply(
+    mean,
+    na.rm = TRUE,
+    before = 2L,
+    after = 1L,
+    memoryOverCPU = FALSE
+  )$values(TRUE)[["col1"]],
   c(2, 3, 4, 6, 8, 10, 12, 13),
   info = '"..." passes on arguments correctly (memoryOverCPU = FALSE)'
 )
 
 expect_identical(
-  DTSg$new(DT1)$rollapply(mean, na.rm = TRUE, before = 0L, after = 0L)$values(TRUE)[["col1"]],
+  DTSg$new(DT1)$rollapply(
+    mean,
+    na.rm = TRUE,
+    before = 0L,
+    memoryOverCPU = TRUE
+  )$values(TRUE)[["col1"]],
   DT1[["col1"]],
   info = "window of size one returns identity (memoryOverCPU = TRUE)"
 )
 
 expect_identical(
-  DTSg$new(DT1)$rollapply(mean, na.rm = TRUE, before = 0L, after = 0L, memoryOverCPU = FALSE)$values(TRUE)[["col1"]],
+  DTSg$new(DT1)$rollapply(
+    mean,
+    na.rm = TRUE,
+    before = 0L,
+    memoryOverCPU = FALSE
+  )$values(TRUE)[["col1"]],
   DT1[["col1"]],
   info = "window of size one returns identity (memoryOverCPU = FALSE)"
 )
@@ -522,14 +640,15 @@ expect_identical(
     weighted.mean,
     na.rm = TRUE,
     before = 2L,
-    after = 1L
+    after = 1L,
+    memoryOverCPU = TRUE
   )$values(TRUE)[["col1"]][3L],
   {
     weights <- 1 / c(rev(seq_len(2) + 1), 1, seq_len(1) + 1)^1
     weights <- weights / sum(weights)
     weighted.mean(DT1[["col1"]][1:4], weights, na.rm = TRUE)
   },
-  info = '"inverseDistance" weights are correct (power = one)'
+  info = '"inverseDistance" weights are correct (power = 1 and memoryOverCPU = TRUE)'
 )
 
 expect_identical(
@@ -546,14 +665,13 @@ expect_identical(
     weights <- weights / sum(weights)
     weighted.mean(DT1[["col1"]][1:4], weights, na.rm = TRUE)
   },
-  info = '"inverseDistance" weights are correct (power = two)'
+  info = '"inverseDistance" weights are correct (power = 2 and memoryOverCPU = FALSE)'
 )
 
 expect_identical(
   DTSg$new(DT1)$rollapply(
     function(x, ...) {identity(x)},
     before = 0L,
-    after = 0L,
     cols = c("col1", "col2"),
     resultCols = c("col1", "col4"),
     suffix = "_character"
@@ -566,7 +684,6 @@ expect_identical(
   DTSg$new(DT1)$rollapply(
     function(x, ...) {identity(x)},
     before = 0L,
-    after = 0L,
     cols = c("col1", "col2"),
     suffix = "_identity"
   )$cols(),
@@ -574,10 +691,121 @@ expect_identical(
   info = '"suffix" adds columns correctly'
 )
 
+#### setCols method ####
+expect_identical(
+  DTSg$new(DT1)$setCols(2L, "col2", 3)$values(TRUE)[["col2"]],
+  DT1[["col1"]],
+  info = "values are set correctly (numeric vector and single column)"
+)
+
+expect_identical(
+  {
+    i <- 1:3
+    DTSg$new(DT1)$setCols(
+      i,
+      c("col1", "col2", "col3"),
+      DT2[1:3, .(col1, as.numeric(col2), col3)],
+    )$values()
+  },
+  setkey(rbind(
+    DT2[1:3, .(date = DT1$date[1:3], col1, col2 = as.numeric(col2), col3)],
+    DT1[4:8]
+  ), "date"),
+  info = "values are set correctly (numeric vector and multiple columns)"
+)
+
+expect_identical(
+  {
+    col2 <- rep(NA_real_, 8)
+    DTSg$new(DT1)$setCols(is.na(col2), "col2", 3)$values(TRUE)[["col2"]]
+  },
+  DT1[["col1"]],
+  info = "values are set correctly (expression)"
+)
+
+expect_identical(
+  DTSg$new(DT1)$setCols(, "col4", DT1[["col1"]])$values(TRUE)[["col4"]],
+  DT1[["col1"]],
+  info = "column is added correctly"
+)
+
+expect_identical(
+  DTSg$new(DT1)$setCols(, "col2", NULL)$cols(),
+  c("col1", "col3"),
+  info = "column is removed correctly"
+)
+
+expect_error(
+  DTSg$new(DT1)$setCols(, ".dateTime", 1),
+  info = "setting .dateTime column returns error"
+)
+
+expect_error(
+  DTSg$new(DT1)$setCols(, c("col1", "col2", "col3"), NULL),
+  info = "removing all value columns returns error"
+)
+
+#### subset method ####
+expect_identical(
+  {
+    i <- 1L
+    DTSg$new(DT1)$subset(i, funby = byYmdH__)$values()
+  },
+  setkey(DT1[c(1L, 3L, 5L, 7L), ], "date"),
+  info = "rows are filtered correctly (numeric vector and funby)"
+)
+
+expect_identical(
+  DTSg$new(DT1)$subset(.N, funby = byYmdH__)$values(),
+  setkey(DT1[c(2L, 4L, 6L, 8L), ], "date"),
+  info = "rows are filtered correctly (expression and funby)"
+)
+
+expect_identical(
+  {
+    col2 <- 1:4
+    DTSg$new(DT1)$subset(col2 > 8, "col2")$values()
+  },
+  setkey(DT1[5:8, .(date, col2)], "date"),
+  info = "rows and columns are subset correctly"
+)
+
+expect_identical(
+  DTSg$new(DT1)$subset(, c("col1", "col3"))$values(),
+  setkey(DT1[, .(date, col1, col3)], "date"),
+  info = "columns are selected correctly"
+)
+
+expect_identical(
+  DTSg$new(DT1)$subset(c(1L, 8L))$values(),
+  setkey(DT1[c(1L, 8L), ], "date"),
+  info = "missing values are made implicit"
+)
+
+expect_error(
+  DTSg$new(DT1)$subset(col2 > 100),
+  info = "empty filter result returns error"
+)
+
+expect_error(
+  DTSg$new(DT1)$subset(12L),
+  info = "missing timestamps return error"
+)
+
+expect_error(
+  DTSg$new(DT1)$subset(c(1L, 1L)),
+  info = "duplicated timestamps return error"
+)
+
+expect_error(
+  DTSg$new(DT1)$subset(, ""),
+  info = '"empty" selection returns error'
+)
+
 #### summary method ####
 expect_identical(
   DTSg$new(DT1)$summary(),
-  summary(DT1[, -1L, with = FALSE]),
+  summary(DT1[, -1L]),
   info = "values are summarised correctly"
 )
 
